package jp.co.thinkit.equinox.hello.ui.internal;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import jp.co.thinkit.equinox.hello.ui.INotificationMessage;

import org.eclipse.jface.dialogs.PopupDialog;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.program.Program;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Link;
import org.eclipse.swt.widgets.Shell;

public class NotificationPopupDialog extends PopupDialog {

	private Rectangle bounds;
	private Composite sectionClient;

	private List<INotificationMessage> contents;
	private Font font;
	
	public NotificationPopupDialog(Shell shell) {
		super(shell, PopupDialog.INFOPOPUP_SHELLSTYLE | SWT.ON_TOP, false, false, false, false, null, null);
	}
	
	public void setContents(List<INotificationMessage> contents) {
		this.contents = contents;
	}


	@Override
	protected Control createContents(Composite parent) {
		Font defaultFont = JFaceResources.getDefaultFont();
		font = new Font(getShell().getDisplay(),defaultFont.getFontData()[0].getName(),16,SWT.BOLD);
		getShell().setBackground(
				getShell().getDisplay().getSystemColor(SWT.COLOR_WHITE));
		return createDialogArea(parent);
	}

	@Override
	protected final Control createDialogArea(final Composite parent) {
		parent.setBackground(getShell().getDisplay().getSystemColor(SWT.COLOR_BLACK));
		getShell().setText("Notification");

		Composite body = new Composite(parent,SWT.NONE);
		body.setBackground(getShell().getDisplay().getSystemColor(SWT.COLOR_WHITE));
		body.setLayout(new GridLayout());
		Label title = new Label(body, SWT.NONE);
		title.setText("Notification");
		title.setBackground(getShell().getDisplay().getSystemColor(SWT.COLOR_WHITE));
		title.setFont(font);
		sectionClient = new Composite(body,SWT.None);
		sectionClient.setBackground(getShell().getDisplay().getSystemColor(SWT.COLOR_WHITE));
		sectionClient.setLayout(new GridLayout(1, false));
		Label date = new Label(sectionClient, SWT.NONE);
		date.setBackground(getShell().getDisplay().getSystemColor(SWT.COLOR_WHITE));
		date.setText(SimpleDateFormat.getDateTimeInstance().format(new Date()));
		if(contents != null){
			for(INotificationMessage content : contents){
				final String url = content.getLink();
				if(url != null){
					Link link = new Link(sectionClient, SWT.None);
					link.setBackground(getShell().getDisplay().getSystemColor(SWT.COLOR_WHITE));
					link.setText("<a href=\"" + url + "\">" + content.getTitle() + "</a>");
					link.addSelectionListener(new SelectionAdapter(){
						@Override
						public void widgetSelected(SelectionEvent e) {
							Program.findProgram(".html").execute(url.toString());
						}
					});
				}else{
					Label label = new Label(sectionClient,SWT.NONE);
					label.setText(content.getTitle());
					label.setBackground(getShell().getDisplay().getSystemColor(SWT.COLOR_WHITE));
				}
			}
		}

		parent.pack();
		return parent;
	}

	@Override
	public void initializeBounds() {
		getShell().setBounds(restoreBounds());
	}

	@Override
	public int open() {
		if(contents.isEmpty()) return CANCEL;
		return super.open();
	}
	
	private Rectangle restoreBounds() {
		bounds = getShell().getBounds();
		Rectangle maxBounds = null;

		// fallback
		Display display = Display.getCurrent();
		if (display == null)
			display = Display.getDefault();
		if (display != null && !display.isDisposed())
			maxBounds = display.getPrimaryMonitor().getClientArea();

		if (bounds.width > -1 && bounds.height > -1) {
			if (maxBounds != null) {
				bounds.width = Math.min(bounds.width, maxBounds.width);
				bounds.height = Math.min(bounds.height, maxBounds.height);
			}
			// Enforce an absolute minimal size
			bounds.width = Math.max(bounds.width, 30);
			bounds.height = Math.max(bounds.height, 30);
		}

		if (bounds.x > -1 && bounds.y > -1 && maxBounds != null) {
			if (bounds.width > -1 && bounds.height > -1) {
				bounds.x = maxBounds.x + maxBounds.width - bounds.width;
				bounds.y = maxBounds.y + maxBounds.height - bounds.height;
			}
		}

		return bounds;
	}
	
	@Override
	public boolean close() {
		font.dispose();
		return super.close();
	}

}
