package jp.co.thinkit.equinox.hello.ui.internal;

import java.util.ArrayList;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;

import jp.co.thinkit.equinox.hello.ui.INotificationMessage;
import jp.co.thinkit.equinox.hello.ui.INotificationService;

import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.util.tracker.ServiceTracker;

public class UIActivator implements BundleActivator {

	private static final int PERIOD = 60 * 5 * 1000;
	private static final int Y_MOVING_HEIGHT = 3;
	private static final int Y_MOVING_DISTANCE = 15;
	private static Bundle bundle;
	private Timer openTimer;
	private NotificationPopupDialog dialog;
	private Display display;
	private Shell shell;
	private ServiceTracker tracker;
		
	public void start(BundleContext context) throws Exception {
		bundle = context.getBundle();
		if(tracker == null){
			tracker = new ServiceTracker(context, INotificationService.class.getName(),null);
		}
		tracker.open(true);
        createOpenTimer();
	}

	private void createOpenTimer() {
		openTimer = new Timer();
		openTimer.scheduleAtFixedRate(new TimerTask(){
			@Override
			public void run() {
				final List<INotificationMessage> contents = new ArrayList<INotificationMessage>();
				Object[] services = tracker.getServices();
				if(services == null) return;
				for(Object obj : services){
					INotificationService service = (INotificationService) obj;
					contents.addAll(service.getContents());
				}
				Display current = Display.getCurrent();
				if(current == null || current.isDisposed()){
					display = new Display();
				}else{
					display = current;
				}

			    display.syncExec(new Runnable(){
			    	public void run() {
						shell = new Shell(display);
					    dialog = new NotificationPopupDialog(shell);
						dialog.setBlockOnOpen(false);
						dialog.setContents(contents);
						if(dialog.open() == NotificationPopupDialog.CANCEL){
							return;
						}
						display.timerExec(30 * 1000, new Runnable(){
							public void run() {
								if(dialog == null) return;
								dialog.close();
								shell.dispose();
							}
						});
						int yLocation = Y_MOVING_DISTANCE;
						Shell dialogShell = dialog.getShell();
						int shellLocationY = dialogShell.getLocation().y + Y_MOVING_DISTANCE;
						dialogShell.setLocation(dialogShell.getLocation().x, 
								shellLocationY);
						while (!shell.isDisposed ()){
							if(yLocation >= 0){
								shellLocationY -= Y_MOVING_HEIGHT;
								dialogShell.setLocation(dialogShell.getLocation().x, 
										shellLocationY);
								try {
									Thread.sleep(70);
								} catch (InterruptedException e) {
								}
								yLocation -= Y_MOVING_HEIGHT;
							}
				            if (!display.readAndDispatch ()) display.sleep ();
				        }
			    	};
			    });
			}
		}
		,2000,PERIOD);
	}

	public void stop(BundleContext context) throws Exception {
		openTimer.cancel();
		openTimer.purge();
		openTimer = null;
		tracker.close();
		tracker = null;
		Display current = Display.getCurrent();
		if(current == null || current.isDisposed()){
			return;
		}
		if(current != null && !current.isDisposed()){
			current.asyncExec(new Runnable(){
				public void run() {
					if(shell != null && !shell.isDisposed()) shell.dispose();
					display.dispose();
				}
			});			
		}
	}

	public static Bundle getBundle() {
		return bundle;
	}

}
